@@ -45,7 +45,9 @@
 #include <linux/crypto.h>
 #include <linux/interrupt.h>
 
-#include <crypto/cryptodev.h>
+#include <opencrypto/crypto.h>
+#include <opencrypto/cryptodev.h>
+
 #include <linux/uio.h>
 
 static int32_t			 null_id = -1;
