@@ -431,7 +431,7 @@ fmt_common_shell_out(char *buf, int blen, struct connection *c
 	strcpy(secure_xauth_username_str, "PLUTO_XAUTH_USERNAME='");
 	
 	len = strlen(secure_xauth_username_str);
-	remove_metachar(st->st_xauth_username
+	remove_metachar((unsigned char *)st->st_xauth_username
 			,secure_xauth_username_str+len
 			,sizeof(secure_xauth_username_str)-(len+2));
 	strncat(secure_xauth_username_str, "'", sizeof(secure_xauth_username_str)-1);
@@ -2122,7 +2122,9 @@ install_inbound_ipsec_sa(struct state *st)
 	    return FALSE;
 	}
     }
+#if 0
     DBG_log("outgoing SA has refhim=%u", st->refhim);
+#endif
 
     /* (attempt to) actually set up the SAs */
     return setup_half_ipsec_sa(st, TRUE);
