@@ -116,7 +116,6 @@ usage(const char *mess)
 	    " \\\n\t"
 	    "[--nofork]"
 	    " [--stderrlog]"
-	    " [--use-nostack]"         /* old --no_klips */
 	    " [--nocrsend]"
 	    " [--strictcrlpolicy]"
 	    " [--crlcheckinterval]"
@@ -128,6 +127,7 @@ usage(const char *mess)
 	    " [--use-mastklips]"
 	    " [--use-bsdkame]"
 	    " [--use-acceleration=<never,ipsec,pk,available>]"
+	    " [--use-nostack]"         /* old --no_klips */
 	    " \\\n\t"
 	    "[--interface <ifname|ifaddr>]"
 	    " [--ikeport <port-number>]"
@@ -136,6 +136,8 @@ usage(const char *mess)
 	    " \\\n\t"
 	    "[--perpeerlogbase <path>] [--perpeerlog]"
 	    " \\\n\t"
+	    "[--coredir <dirname>] [--noretransmits]"
+	    " \\\n\t"
 	    "[--secretsfile <secrets-file>]"
 	    " [--ipsecdir <ipsec-dir>]"
 	    " \\\n\t"
@@ -153,9 +155,11 @@ usage(const char *mess)
 	    " [--debug-x509]"
 	    " \\\n\t"
 	    "[--debug-control]"
+	    "[--debug-lifecycle]"
 	    " [--debug-klips]"
 	    " [--debug-x509]"
 	    " [--debug-dns]"
+	    " [--debug-oppo]"
 	    " [--debug-dpd]"
 	    " [ --debug-private]"
 	    " [ --debug-pfkey]"
